@@ -2,6 +2,8 @@ package org.raku.rakudo;
 
 import java.util.*;
 
+import it.unimi.dsi.fastutil.objects.Object2IntOpenHashMap;
+
 import org.raku.nqp.runtime.*;
 import org.raku.nqp.sixmodel.*;
 import org.raku.nqp.sixmodel.reprs.ContextRefInstance;
@@ -154,9 +156,9 @@ public final class Binder {
             int paramFlags, SixModelObject attrPackage, SixModelObject value, Object[] error) {
         /* Find self. */
         StaticCodeInfo sci = cf.codeRef.staticInfo;
-        Integer selfIdx = sci.oTryGetLexicalIdx("self");
+        int selfIdx = sci.oTryGetLexicalIdx("self");
         SixModelObject self = null;
-        if (selfIdx == null) {
+        if (selfIdx == -1) {
             self = Ops.getlexouter("self", tc);
             if (self == null) {
                 if (error != null)
@@ -177,13 +179,8 @@ public final class Binder {
                since *trying* to get a container would throw already, we first check
                if the target Attribute is native. */
             int hint = -1;
-            for (HashMap<String, Integer> map : ((P6OpaqueREPRData) (attrPackage.st.REPRData)).nameToHintMap) {
-                try {
-                    hint = map.get(varName);
-                }
-                catch (Exception e) {
-                    continue;
-                }
+            for (Object2IntOpenHashMap<String> map : ((P6OpaqueREPRData) (attrPackage.st.REPRData)).nameToHintMap) {
+                hint = map.getOrDefault(varName, -1);
             }
             REPR attrREPR = null;
             if (((P6OpaqueREPRData) (attrPackage.st.REPRData)).flattenedSTables[hint] != null) {
@@ -470,7 +467,7 @@ public final class Binder {
                     /* Try to figure out the most helpful name for the expected. */
                     SixModelObject expectedType = null;
                     SixModelObject postConstraints = param.get_attribute_boxed(tc, gcx.Parameter,
-                            "$!post_contraints", HINT_post_constraints);
+                            "$!post_constraints", HINT_post_constraints);
                     if (postConstraints != null) {
                         SixModelObject consType = postConstraints.at_pos_boxed(tc, 0);
                         expectedType = (Ops.istype(consType, gcx.Code, tc) != 0)
@@ -688,7 +685,7 @@ public final class Binder {
         /* Handle any constraint types (note that they may refer to the parameter by
          * name, so we need to have bound it already). */
         SixModelObject postConstraints = param.get_attribute_boxed(tc, gcx.Parameter,
-            "$!post_contraints", HINT_post_constraints);
+            "$!post_constraints", HINT_post_constraints);
         if (postConstraints != null) {
             long numConstraints = postConstraints.elems(tc);
             for (long i = 0; i < numConstraints; i++) {
@@ -819,6 +816,12 @@ public final class Binder {
         return exploder.explodeFlattening(tc.curFrame, new Object[] { list, hash });
     }
 
+    private static final CallSiteDescriptor parameterizeArray = new CallSiteDescriptor(
+        new byte[] { CallSiteDescriptor.ARG_OBJ, CallSiteDescriptor.ARG_OBJ, CallSiteDescriptor.ARG_OBJ }, null);
+    private static final CallSiteDescriptor parameterizeHash = new CallSiteDescriptor(
+        new byte[] { CallSiteDescriptor.ARG_OBJ, CallSiteDescriptor.ARG_OBJ,
+            CallSiteDescriptor.ARG_OBJ, CallSiteDescriptor.ARG_OBJ
+        }, null);
     /* This takes a signature element and either runs the closure to get a default
      * value if there is one, or creates an appropriate undefined-ish thingy. */
     private static SixModelObject handleOptional(ThreadContext tc, RakOps.GlobalExt gcx, int flags, SixModelObject param, CallFrame cf) {
@@ -828,8 +831,8 @@ public final class Binder {
             String varName = tc.native_s;
             CallFrame curOuter = cf.outer;
             while (curOuter != null) {
-                Integer idx = curOuter.codeRef.staticInfo.oTryGetLexicalIdx(varName);
-                if (idx != null)
+                int idx = curOuter.codeRef.staticInfo.oTryGetLexicalIdx(varName);
+                if (idx != -1)
                     return curOuter.oLex[idx];
                 curOuter = curOuter.outer;
             }
@@ -853,11 +856,61 @@ public final class Binder {
         /* Otherwise, go by sigil to pick the correct default type of value. */
         else {
             if ((flags & SIG_ELEM_ARRAY_SIGIL) != 0) {
-                SixModelObject res = gcx.Array.st.REPR.allocate(tc, gcx.Array.st);
+                SixModelObject paramType = param.get_attribute_boxed(tc, gcx.Parameter, "$!type", HINT_type);
+                SixModelObject defaultType = null;
+
+                if (paramType == gcx.Positional) {
+                    defaultType = gcx.Array;
+                }
+                else {
+                    /* TODO: Find clean solution for handling @deprecation
+                     * during compiliation of setting. */
+                    param.get_attribute_native(tc, gcx.Parameter, "$!variable_name", HINT_variable_name);
+                    String varName = tc.native_s;
+                    if (varName.equals("@deprecation")) {
+                        SixModelObject compilingCoreSetting = Ops.getlexdyn("$*COMPILING_CORE_SETTING", tc);
+                        if (Ops.isnull(compilingCoreSetting) == 0)
+                            defaultType = gcx.Array;
+                    }
+
+                    if (defaultType == null) {
+                        SixModelObject ofMeth = Ops.findmethod(paramType, "of", tc);
+                        Ops.invokeDirect(tc, ofMeth, Ops.invocantCallSite, new Object[] { paramType });
+                        SixModelObject ofType = Ops.result_o(tc.curFrame);
+
+                        SixModelObject arrayHOW = gcx.Array.st.HOW;
+                        SixModelObject parameterizeMeth = Ops.findmethod(arrayHOW, "parameterize", tc);
+                        Ops.invokeDirect(tc, parameterizeMeth, parameterizeArray, new Object[] { arrayHOW, gcx.Array, ofType });
+                        defaultType = Ops.result_o(tc.curFrame);
+                    }
+                }
+
+                SixModelObject res = Ops.create(defaultType, tc);
                 return res;
             }
             else if ((flags & SIG_ELEM_HASH_SIGIL) != 0) {
-                SixModelObject res = gcx.Hash.st.REPR.allocate(tc, gcx.Hash.st);
+                SixModelObject paramType = param.get_attribute_boxed(tc, gcx.Parameter, "$!type", HINT_type);
+                SixModelObject defaultType;
+
+                if (paramType == gcx.Associative) {
+                    defaultType = gcx.Hash;
+                }
+                else {
+                    SixModelObject ofMeth = Ops.findmethod(paramType, "of", tc);
+                    Ops.invokeDirect(tc, ofMeth, Ops.invocantCallSite, new Object[] { paramType });
+                    SixModelObject ofType = Ops.result_o(tc.curFrame);
+
+                    SixModelObject keyofMeth = Ops.findmethod(paramType, "keyof", tc);
+                    Ops.invokeDirect(tc, keyofMeth, Ops.invocantCallSite, new Object[] { paramType });
+                    SixModelObject keyofType = Ops.result_o(tc.curFrame);
+
+                    SixModelObject hashHOW = gcx.Hash.st.HOW;
+                    SixModelObject parameterizeMeth = Ops.findmethod(hashHOW, "parameterize", tc);
+                    Ops.invokeDirect(tc, parameterizeMeth, parameterizeHash, new Object[] { hashHOW, gcx.Hash, ofType, keyofType });
+                    defaultType = Ops.result_o(tc.curFrame);
+                }
+
+                SixModelObject res = Ops.create(defaultType, tc);
                 return res;
             }
             else {
@@ -914,9 +967,9 @@ public final class Binder {
          * to work on. We'll delete stuff from it as we bind, and what we have
          * left over can become the slurpy hash or - if we aren't meant to be
          * taking one - tell us we have a problem. */
-        HashMap<String, Integer> namedArgsCopy = csd.nameMap == null
+        Object2IntOpenHashMap<String> namedArgsCopy = csd.nameMap == null
             ? null
-            : new HashMap<String, Integer>(csd.nameMap);
+            : new Object2IntOpenHashMap<String>(csd.nameMap);
 
         /* Now we'll walk through the signature and go about binding things. */
         int numPosArgs = csd.numPositionals;
@@ -1078,20 +1131,21 @@ public final class Binder {
             /* Else, it's a non-slurpy named. */
             else {
                 /* Try and get hold of value. */
-                Integer lookup = null;
+                int lookup = -1;
                 if (namedArgsCopy != null) {
                     long numNames = namedNames.elems(tc);
                     for (long j = 0; j < numNames; j++) {
                         namedNames.at_pos_native(tc, j);
                         String name = tc.native_s;
-                        lookup = namedArgsCopy.remove(name);
-                        if (lookup != null)
+                        if (namedArgsCopy.containsKey(name)) {
+                            lookup = namedArgsCopy.remove(name);
                             break;
+                        }
                     }
                 }
 
                 /* Did we get one? */
-                if (lookup == null) {
+                if (lookup == -1) {
                     /* Nope. We'd better hope this param was optional... */
                     if ((flags & SIG_ELEM_IS_OPTIONAL) != 0) {
                         bindFail = bindOneParam(tc, gcx, cf, param,
@@ -1131,8 +1185,9 @@ public final class Binder {
             if (error != null) {
                 int numExtra = namedArgsCopy.size();
                 if (numExtra == 1) {
-                    for (String name : namedArgsCopy.keySet())
+                    for (String name : namedArgsCopy.keySet()) {
                         error[0] = "Unexpected named argument '" + name + "' passed";
+                    }
                 }
                 else {
                     boolean first = true;
@@ -1155,13 +1210,13 @@ public final class Binder {
     }
 
     /* Takes any nameds we didn't capture yet and makes a VM Hash of them. */
-    private static SixModelObject vmHashOfRemainingNameds(ThreadContext tc, RakOps.GlobalExt gcx, HashMap<String, Integer> namedArgsCopy, Object[] args) {
+    private static SixModelObject vmHashOfRemainingNameds(ThreadContext tc, RakOps.GlobalExt gcx, Object2IntOpenHashMap<String> namedArgsCopy, Object[] args) {
         SixModelObject slurpy = gcx.Mu;
         if (namedArgsCopy != null) {
             SixModelObject BOOTHash = tc.gc.BOOTHash;
             slurpy = BOOTHash.st.REPR.allocate(tc, BOOTHash.st);
             for (String name : namedArgsCopy.keySet()) {
-                int lookup = namedArgsCopy.get(name);
+                int lookup = namedArgsCopy.getInt(name);
                 switch (lookup & 7) {
                 case CallSiteDescriptor.ARG_OBJ:
                     slurpy.bind_key_boxed(tc, name, (SixModelObject)args[lookup >> 6]);
